"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AggregateQueryParam = void 0;
exports.getAggregatedFields = getAggregatedFields;
const tslib_1 = require("tslib");
const common_1 = require("@nestjs/common");
const graphql_1 = require("@nestjs/graphql");
const graphql_fields_1 = tslib_1.__importDefault(require("graphql-fields"));
const EXCLUDED_FIELDS = ['__typename'];
const QUERY_OPERATORS = ['groupBy', 'count', 'distinctCount', 'avg', 'sum', 'min', 'max'];
function getAggregatedFields(fields) {
    return QUERY_OPERATORS.filter((operator) => !!fields[operator]).reduce((query, operator) => {
        const queryFields = Object.entries(fields[operator]);
        const queryFieldsWithRelations = queryFields.map(([key, value]) => {
            const relations = Object.keys(value);
            if (relations.length)
                return {
                    [key]: relations,
                };
            return key;
        });
        if (queryFieldsWithRelations.length) {
            return { ...query, [operator]: queryFieldsWithRelations };
        }
        return query;
    }, {});
}
exports.AggregateQueryParam = (0, common_1.createParamDecorator)((data, ctx) => {
    const info = graphql_1.GqlExecutionContext.create(ctx).getInfo();
    // eslint-disable-next-line @typescript-eslint/no-unsafe-call,@typescript-eslint/no-unsafe-argument
    const fields = (0, graphql_fields_1.default)(info, {}, { excludedFields: EXCLUDED_FIELDS });
    return getAggregatedFields(fields);
});
//# sourceMappingURL=aggregate-query-param.decorator.js.map