import { FieldOptions, ReturnTypeFunc } from '@nestjs/graphql';
import { Class, FilterComparisonOperators } from '@rezonate/nestjs-query-core';
export type FilterableFieldOptions<T = any> = {
    allowedComparisons?: FilterComparisonOperators<unknown>[];
    isJSON?: boolean;
    filterRequired?: boolean;
    filterOnly?: boolean;
    futureDate?: boolean;
} & FieldOptions<T>;
export interface FilterableFieldDescriptor<T = any> {
    propertyName: string;
    target: Class<unknown>;
    returnTypeFunc?: ReturnTypeFunc;
    advancedOptions?: FilterableFieldOptions<T>;
}
export interface FilterableRelationFields<T = any> extends FilterableFieldDescriptor<T> {
    propertyName: string;
    relationPropertyName: string;
    DTO: Class<unknown>;
}
/**
 * Decorator for Fields that should be filterable through a [[FilterType]]
 *
 * @example
 *
 * In the following DTO `id`, `title` and `completed` are filterable.
 *
 * ```ts
 * import { FilterableField } from '@rezonate/nestjs-query-graphql';
 * import { ObjectType, ID, GraphQLISODateTime, Field } from '@nestjs/graphql';
 *
 * @ObjectType('TodoItem')
 * export class TodoItemDTO {
 *   @FilterableField(() => ID)
 *   id!: string;
 *
 *   @FilterableField()
 *   title!: string;
 *
 *   @FilterableField()
 *   completed!: boolean;
 *
 *   @Field(() => GraphQLISODateTime)
 *   created!: Date;
 *
 *   @Field(() => GraphQLISODateTime)
 *   updated!: Date;
 * }
 * ```
 */
export declare function FilterableField(): PropertyDecorator & MethodDecorator;
export declare function FilterableField(options: FilterableFieldOptions): PropertyDecorator & MethodDecorator;
export declare function FilterableField(returnTypeFunction?: ReturnTypeFunc, options?: FilterableFieldOptions): PropertyDecorator & MethodDecorator;
export declare function getFilterableFields<DTO>(DTOClass: Class<DTO>): FilterableFieldDescriptor[];
export declare function getRelatedDTOsMap<DTO>(DTOClass: Class<DTO>): Map<string, import("../resolvers/relations/relations.interface").ResolverOneRelation<unknown>>;
export declare function getFilterableRelationFields<DTO>(DTOClass: Class<DTO>, realtionTypes: ('one' | 'many')[]): FilterableRelationFields[];
export declare function getFilterableRelationFieldsNames<DTO>(DTOClass: Class<DTO>, DTOFieldsNames: string[], realtionTypes: ('one' | 'many')[]): string[];
