"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NestjsQueryGraphQLModule = void 0;
const providers_1 = require("./providers");
const subscription_1 = require("./subscription");
const types_1 = require("./types");
const relative_date_scalar_type_1 = require("./types/relative-date-scalar.type");
const relative_date_future_scalar_type_1 = require("./types/relative-date-future-scalar.type");
class NestjsQueryGraphQLModule {
    static forFeature(opts) {
        const providers = this.getProviders(opts);
        return {
            module: NestjsQueryGraphQLModule,
            imports: [...opts.imports],
            providers: [...providers],
            exports: [...providers, ...opts.imports],
        };
    }
    static defaultPubSubProvider() {
        return { provide: (0, subscription_1.pubSubToken)(), useValue: (0, subscription_1.defaultPubSub)() };
    }
    static getProviders(opts) {
        return [
            ...this.getServicesProviders(opts),
            ...this.getPubSubProviders(opts),
            ...this.getAuthorizerProviders(opts),
            ...this.getHookProviders(opts),
            ...this.getResolverProviders(opts),
            ...this.getScalarsProviders(),
        ];
    }
    static getScalarsProviders() {
        return [types_1.ConnectionCursorScalar, relative_date_scalar_type_1.RelativeDateScalar, relative_date_future_scalar_type_1.RelativeDateScalarFuture];
    }
    static getPubSubProviders(opts) {
        return [opts.pubSub ?? this.defaultPubSubProvider()];
    }
    static getServicesProviders(opts) {
        return opts.services ?? [];
    }
    static getResolverProviders(opts) {
        return (0, providers_1.createResolvers)(opts.resolvers ?? []);
    }
    static getAuthorizerProviders(opts) {
        const resolverDTOs = opts.resolvers?.map((r) => r.DTOClass) ?? [];
        const dtos = opts.dtos?.map((o) => o.DTOClass) ?? [];
        return (0, providers_1.createAuthorizerProviders)([...resolverDTOs, ...dtos]);
    }
    static getHookProviders(opts) {
        return (0, providers_1.createHookProviders)([...(opts.resolvers ?? []), ...(opts.dtos ?? [])]);
    }
}
exports.NestjsQueryGraphQLModule = NestjsQueryGraphQLModule;
//# sourceMappingURL=module.js.map