import { Class, Filter, QueryService } from '@rezonate/nestjs-query-core';
import { ConnectionOptions, FindOneArgsType, InferConnectionTypeFromStrategy, PagingStrategies, QueryArgsTypeOpts } from '../types';
import { CursorQueryArgsTypeOpts, QueryType, StaticQueryType } from '../types/query';
import { ExtractPagingStrategy, ResolverClass, ResolverOpts, ServiceResolver } from './resolver.interface';
import { FilterConstructor } from '../types/query/filter.type';
export type ReadResolverFromOpts<DTO, Opts extends ReadResolverOpts<DTO>, QS extends QueryService<DTO>> = ReadResolver<DTO, ExtractPagingStrategy<DTO, Opts>, QS>;
export type ReadResolverOpts<DTO> = {
    QueryArgs?: StaticQueryType<DTO, PagingStrategies>;
    idField?: keyof DTO;
} & ResolverOpts & QueryArgsTypeOpts<DTO> & Pick<ConnectionOptions, 'enableTotalCount'>;
export interface ReadResolver<DTO, PS extends PagingStrategies, QS extends QueryService<DTO>> extends ServiceResolver<DTO, QS> {
    queryMany(query: QueryType<DTO, PagingStrategies>, authorizeFilter?: Filter<DTO>): Promise<InferConnectionTypeFromStrategy<DTO, PS>>;
    queryManyToCSV(query: QueryType<DTO, PagingStrategies>, authorizeFilter?: Filter<DTO>): Promise<string>;
    queryManyIds(query: QueryType<DTO, PagingStrategies>, authorizeFilter?: Filter<DTO>): Promise<string[]>;
    findById(id: FindOneArgsType, authorizeFilter?: Filter<DTO>): Promise<DTO | undefined>;
}
export declare const getQueryArgs: <DTO>(DTOClass: Class<DTO>) => FilterConstructor<DTO>;
export type QueryArgsField<T extends FilterConstructor<any>> = InstanceType<T>;
/**
 * @internal
 * Mixin to add `read` graphql endpoints.
 */
export declare const Readable: <DTO, ReadOpts extends ReadResolverOpts<DTO>, QS extends QueryService<DTO>>(DTOClass: Class<DTO>, opts: ReadOpts) => <B extends Class<ServiceResolver<DTO, QS>>>(BaseClass: B) => Class<ReadResolverFromOpts<DTO, ReadOpts, QS>> & B;
export declare const ReadResolverFactory: <DTO, ReadOpts extends ReadResolverOpts<DTO> = CursorQueryArgsTypeOpts<DTO>, QS extends QueryService<DTO> = QueryService<DTO>>(DTOClass: Class<DTO>, opts?: ReadOpts) => ResolverClass<DTO, QS, ReadResolverFromOpts<DTO, ReadOpts, QS>>;
