"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReadResolverFactory = exports.Readable = exports.getQueryArgs = void 0;
const tslib_1 = require("tslib");
// eslint-disable-next-line max-classes-per-file
const graphql_1 = require("@nestjs/graphql");
const nestjs_query_core_1 = require("@rezonate/nestjs-query-core");
const lodash_omit_1 = tslib_1.__importDefault(require("lodash.omit"));
const papaparse_1 = tslib_1.__importDefault(require("papaparse"));
const auth_1 = require("../auth");
const common_1 = require("../common");
const decorators_1 = require("../decorators");
const hooks_1 = require("../hooks");
const interceptors_1 = require("../interceptors");
const types_1 = require("../types");
const query_args_1 = require("../types/query/query-args");
const resolver_interface_1 = require("./resolver.interface");
const QUERY_ARGS_TOKEN = Symbol('QUERY_ARGS_TOKEN');
const getQueryArgs = (DTOClass) => (0, types_1.FilterType)(DTOClass);
exports.getQueryArgs = getQueryArgs;
const serializeNestedObjects = (obj) => {
    const result = {};
    Object.entries(obj).forEach(([key, value]) => {
        if (typeof value === 'object' && value !== null) {
            result[key] = JSON.stringify(value);
        }
        else {
            result[key] = value;
        }
    });
    return result;
};
/**
 * @internal
 * Mixin to add `read` graphql endpoints.
 */
const Readable = (DTOClass, opts) => (BaseClass) => {
    const { baseNameLower, pluralBaseNameLower, baseName } = (0, common_1.getDTONames)(DTOClass, opts);
    const readOneQueryName = opts.one?.name ?? baseNameLower;
    const readManyQueryName = opts.many?.name ?? pluralBaseNameLower;
    const { QueryArgs = (0, types_1.QueryArgsType)(DTOClass, { ...opts, connectionName: `${baseName}Connection` }) } = opts;
    const { ConnectionType } = QueryArgs;
    Reflect.defineMetadata(QUERY_ARGS_TOKEN, QueryArgs, DTOClass);
    const commonResolverOpts = (0, lodash_omit_1.default)(opts, 'dtoName', 'one', 'many', 'QueryArgs', 'Connection', 'withDeleted');
    let QA = class QA extends QueryArgs {
    };
    QA = tslib_1.__decorate([
        (0, graphql_1.ArgsType)()
    ], QA);
    let FO = class FO extends (0, types_1.FindOneArgsType)(DTOClass) {
    };
    FO = tslib_1.__decorate([
        (0, graphql_1.ArgsType)()
    ], FO);
    let ReadResolverBase = class ReadResolverBase extends BaseClass {
        async findById(input, authorizeFilter) {
            return this.service.getById(input.id, {
                filter: authorizeFilter,
                withDeleted: opts?.one?.withDeleted,
            });
        }
        async queryMany(query, authorizeFilter) {
            return ConnectionType.createFromPromise((q) => this.service.query(q), (0, nestjs_query_core_1.mergeQuery)(query, { filter: authorizeFilter }), (filter) => this.service.count(filter));
        }
        async queryManyToCSV(query, authorizeFilter) {
            const limitValue = (0, decorators_1.getQueryOptions)(DTOClass).CSVPageLimit || query_args_1.DEFAULT_QUERY_OPTS.CSVPageLimit;
            const res = await this.service.query({ ...(0, nestjs_query_core_1.mergeQuery)(query, { filter: authorizeFilter }), paging: { ...query.paging, limit: limitValue } });
            return papaparse_1.default.unparse(res.map((r) => serializeNestedObjects(r)));
        }
        queryManyIds(query, authorizeFilter) {
            return this.service.queryIds((0, nestjs_query_core_1.mergeQuery)(query, { filter: authorizeFilter }), opts.idField);
        }
    };
    tslib_1.__decorate([
        (0, decorators_1.ResolverQuery)(() => DTOClass, { name: readOneQueryName, description: opts?.one?.description }, commonResolverOpts, { interceptors: [(0, interceptors_1.HookInterceptor)(hooks_1.HookTypes.BEFORE_FIND_ONE, DTOClass), (0, interceptors_1.AuthorizerInterceptor)(DTOClass)] }, opts.one ?? {}),
        tslib_1.__param(0, (0, decorators_1.HookArgs)()),
        tslib_1.__param(1, (0, decorators_1.AuthorizerFilter)({
            operationGroup: auth_1.OperationGroup.READ,
            many: false,
        })),
        tslib_1.__metadata("design:type", Function),
        tslib_1.__metadata("design:paramtypes", [FO, Object]),
        tslib_1.__metadata("design:returntype", Promise)
    ], ReadResolverBase.prototype, "findById", null);
    tslib_1.__decorate([
        (0, decorators_1.ResolverQuery)(() => QueryArgs.ConnectionType.resolveType, { name: readManyQueryName, description: opts?.many?.description }, commonResolverOpts, { interceptors: [(0, interceptors_1.HookInterceptor)(hooks_1.HookTypes.BEFORE_QUERY_MANY, DTOClass), (0, interceptors_1.AuthorizerInterceptor)(DTOClass)] }, opts.many ?? {}),
        tslib_1.__param(0, (0, decorators_1.HookArgs)()),
        tslib_1.__param(1, (0, decorators_1.AuthorizerFilter)({
            operationGroup: auth_1.OperationGroup.READ,
            many: true,
        })),
        tslib_1.__metadata("design:type", Function),
        tslib_1.__metadata("design:paramtypes", [QA, Object]),
        tslib_1.__metadata("design:returntype", Promise)
    ], ReadResolverBase.prototype, "queryMany", null);
    tslib_1.__decorate([
        (0, decorators_1.ResolverQuery)(() => String, { name: `${readManyQueryName}CSV`, description: opts?.many?.description }, commonResolverOpts, { interceptors: [(0, interceptors_1.HookInterceptor)(hooks_1.HookTypes.BEFORE_QUERY_MANY, DTOClass), (0, interceptors_1.AuthorizerInterceptor)(DTOClass)] }, opts.many ?? {}),
        tslib_1.__param(0, (0, decorators_1.HookArgs)()),
        tslib_1.__param(1, (0, decorators_1.AuthorizerFilter)({
            operationGroup: auth_1.OperationGroup.READ,
            many: true,
        })),
        tslib_1.__metadata("design:type", Function),
        tslib_1.__metadata("design:paramtypes", [QA, Object]),
        tslib_1.__metadata("design:returntype", Promise)
    ], ReadResolverBase.prototype, "queryManyToCSV", null);
    tslib_1.__decorate([
        (0, decorators_1.ResolverQuery)(() => [String], { name: `${readOneQueryName}Ids`, description: opts?.many?.description }, commonResolverOpts, {
            interceptors: [(0, interceptors_1.HookInterceptor)(hooks_1.HookTypes.BEFORE_QUERY_MANY, DTOClass), (0, interceptors_1.AuthorizerInterceptor)(DTOClass)],
            disabled: !opts.idField,
        }, opts.many ?? {}),
        tslib_1.__param(0, (0, decorators_1.HookArgs)()),
        tslib_1.__param(1, (0, decorators_1.AuthorizerFilter)({
            operationGroup: auth_1.OperationGroup.READ,
            many: true,
        })),
        tslib_1.__metadata("design:type", Function),
        tslib_1.__metadata("design:paramtypes", [QA, Object]),
        tslib_1.__metadata("design:returntype", void 0)
    ], ReadResolverBase.prototype, "queryManyIds", null);
    ReadResolverBase = tslib_1.__decorate([
        (0, graphql_1.Resolver)(() => DTOClass, { isAbstract: true })
    ], ReadResolverBase);
    return ReadResolverBase;
};
exports.Readable = Readable;
const ReadResolverFactory = (DTOClass, opts = {}) => (0, exports.Readable)(DTOClass, opts)(resolver_interface_1.BaseServiceResolver);
exports.ReadResolverFactory = ReadResolverFactory;
//# sourceMappingURL=read.resolver.js.map