import { Class, DeepPartial, DeleteManyResponse, Filter, QueryService, UpdateManyResponse } from '@rezonate/nestjs-query-core';
import { MutationArgsType, SubscriptionArgsType, UpdateManyInputType, UpdateOneInputType } from '../types';
import { ResolverClass, ServiceResolver, SubscriptionResolverOpts } from './resolver.interface';
export type UpdatedEvent<DTO> = {
    [eventName: string]: DTO;
};
export interface UpdateResolverOpts<DTO> extends SubscriptionResolverOpts {
    UpdateDTOClass?: Class<DeepPartial<DTO>>;
    UpdateOneInput?: Class<UpdateOneInputType<DTO>>;
    UpdateManyInput?: Class<UpdateManyInputType<DTO>>;
}
export interface UpdateResolver<DTO, QS extends QueryService<DTO>> extends ServiceResolver<DTO, QS> {
    updateOne(input: MutationArgsType<UpdateOneInputType<DTO>>, authFilter?: Filter<DTO>): Promise<DTO>;
    updateMany(input: MutationArgsType<UpdateManyInputType<DTO>>, authFilter?: Filter<DTO>): Promise<UpdateManyResponse>;
    updatedOneSubscription(input?: SubscriptionArgsType<DTO>, authFilter?: Filter<DTO>): AsyncIterator<UpdatedEvent<DTO>>;
    updatedManySubscription(authFilter?: Filter<DTO>): AsyncIterator<UpdatedEvent<DeleteManyResponse>>;
}
/**
 * @internal
 * Mixin to add `update` graphql endpoints.
 */
export declare const Updateable: <DTO, QS extends QueryService<DTO>>(DTOClass: Class<DTO>, opts: UpdateResolverOpts<DTO>) => <B extends Class<ServiceResolver<DTO, QS>>>(BaseClass: B) => Class<UpdateResolver<DTO, QS>> & B;
export declare const UpdateResolver: <DTO, QS extends QueryService<DTO> = QueryService<DTO>>(DTOClass: Class<DTO>, opts?: UpdateResolverOpts<DTO>) => ResolverClass<DTO, QS, UpdateResolver<DTO, QS>>;
