"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AggregateResponseType = AggregateResponseType;
const tslib_1 = require("tslib");
// eslint-disable-next-line max-classes-per-file
const graphql_1 = require("@nestjs/graphql");
const nestjs_query_core_1 = require("@rezonate/nestjs-query-core");
const common_1 = require("../../common");
const decorators_1 = require("../../decorators");
const filterable_field_decorator_1 = require("../../decorators/filterable-field.decorator");
const memoMap = new Map();
const getAggregatedType = (name, createFn) => {
    if (memoMap.has(name))
        return memoMap.get(name);
    const result = createFn();
    memoMap.set(name, result);
    return result;
};
const reflector = new nestjs_query_core_1.MapReflector('nestjs-query:aggregate-response-type');
const NumberAggregatedType = (name, fields, NumberType, relationFields = new Map()) => {
    const fieldNames = fields.map((f) => f.propertyName);
    const Aggregated = getAggregatedType(name, () => {
        let AggregatedClass = class AggregatedClass {
        };
        AggregatedClass = tslib_1.__decorate([
            (0, graphql_1.ObjectType)(name),
            (0, graphql_1.Directive)('@shareable')
        ], AggregatedClass);
        return AggregatedClass;
    });
    fieldNames.forEach((propertyName) => {
        (0, graphql_1.Field)(() => NumberType, { nullable: true })(Aggregated.prototype, propertyName);
    });
    relationFields.forEach((related, key) => {
        const objName = (0, common_1.getGraphqlObjectName)(related.DTO, 'Unable to make AggregationResponseType.');
        const rFields = (0, decorators_1.getFilterableFields)(related.DTO);
        const rt = NumberAggregatedType(`${objName}Aggregate`, rFields, NumberType);
        (0, graphql_1.Field)(() => rt, { nullable: true })(Aggregated.prototype, key);
    });
    return Aggregated;
};
const AggregateGroupByType = (name, fields, relationFields = new Map()) => {
    const Aggregated = getAggregatedType(name, () => {
        let AggregatedClass = class AggregatedClass {
        };
        AggregatedClass = tslib_1.__decorate([
            (0, graphql_1.ObjectType)(name),
            (0, graphql_1.Directive)('@shareable')
        ], AggregatedClass);
        return AggregatedClass;
    });
    fields.forEach(({ propertyName, target, returnTypeFunc }) => {
        let rt = returnTypeFunc ? returnTypeFunc() : target;
        if (Array.isArray(rt))
            [rt] = rt;
        (0, graphql_1.Field)(() => rt, { nullable: true })(Aggregated.prototype, propertyName);
    });
    relationFields.forEach((related, key) => {
        const objName = (0, common_1.getGraphqlObjectName)(related.DTO, 'Unable to make AggregationResponseType.');
        const rFields = (0, decorators_1.getFilterableFields)(related.DTO);
        const rt = AggregateGroupByType(`${objName}GroupByAggregate`, rFields);
        (0, graphql_1.Field)(() => rt, { nullable: true })(Aggregated.prototype, key);
    });
    return Aggregated;
};
const AggregatedType = (name, fields, relationFields = new Map()) => {
    const Aggregated = getAggregatedType(name, () => {
        let AggregatedClass = class AggregatedClass {
        };
        AggregatedClass = tslib_1.__decorate([
            (0, graphql_1.ObjectType)(name),
            (0, graphql_1.Directive)('@shareable')
        ], AggregatedClass);
        return AggregatedClass;
    });
    fields.forEach(({ propertyName, target, returnTypeFunc }) => {
        const rt = returnTypeFunc ? returnTypeFunc() : target;
        (0, graphql_1.Field)(() => rt, { nullable: true })(Aggregated.prototype, propertyName);
    });
    relationFields.forEach((related, key) => {
        const objName = (0, common_1.getGraphqlObjectName)(related.DTO, 'Unable to make AggregationResponseType.');
        const rFields = (0, decorators_1.getFilterableFields)(related.DTO);
        const rt = AggregatedType(`${objName}MinMaxAggregate`, rFields);
        (0, graphql_1.Field)(() => rt, { nullable: true })(Aggregated.prototype, key);
    });
    return Aggregated;
};
function AggregateResponseType(DTOClass, opts) {
    const objName = (0, common_1.getGraphqlObjectName)(DTOClass, 'Unable to make AggregationResponseType.');
    const prefix = opts?.prefix ?? objName;
    const isShareable = (0, decorators_1.getShareableDTO)(DTOClass);
    const aggName = `${prefix}AggregateResponse`;
    return reflector.memoize(DTOClass, aggName, () => {
        const fields = (0, decorators_1.getFilterableFields)(DTOClass);
        const relatedDTOs = (0, filterable_field_decorator_1.getRelatedDTOsMap)(DTOClass);
        if (!fields.length && !relatedDTOs.size) {
            throw new Error(`No fields found to create AggregationResponseType for ${DTOClass.name}. Ensure fields are annotated with @FilterableField`);
        }
        const numberFields = fields.filter(({ target }) => target === Number);
        const minMaxFields = fields.filter(({ target }) => target !== Boolean);
        const GroupType = AggregateGroupByType(`${prefix}AggregateGroupBy`, fields, relatedDTOs);
        const CountType = NumberAggregatedType(`${prefix}CountAggregate`, fields, graphql_1.Int, relatedDTOs);
        const FloatType = NumberAggregatedType(`${prefix}FloatAggregate`, numberFields, graphql_1.Float, relatedDTOs);
        const MinMaxType = AggregatedType(`${prefix}MinMaxAggregate`, minMaxFields, relatedDTOs);
        let AggResponse = class AggResponse {
        };
        tslib_1.__decorate([
            (0, graphql_1.Field)(() => GroupType, { nullable: true }),
            tslib_1.__metadata("design:type", Object)
        ], AggResponse.prototype, "groupBy", void 0);
        tslib_1.__decorate([
            (0, graphql_1.Field)(() => CountType, { nullable: true }),
            tslib_1.__metadata("design:type", Object)
        ], AggResponse.prototype, "count", void 0);
        tslib_1.__decorate([
            (0, graphql_1.Field)(() => CountType, { nullable: true }),
            tslib_1.__metadata("design:type", Object)
        ], AggResponse.prototype, "distinctCount", void 0);
        tslib_1.__decorate([
            (0, decorators_1.SkipIf)(() => numberFields.length === 0, (0, graphql_1.Field)(() => FloatType, { nullable: true })),
            tslib_1.__metadata("design:type", Object)
        ], AggResponse.prototype, "sum", void 0);
        tslib_1.__decorate([
            (0, decorators_1.SkipIf)(() => numberFields.length === 0, (0, graphql_1.Field)(() => FloatType, { nullable: true })),
            tslib_1.__metadata("design:type", Object)
        ], AggResponse.prototype, "avg", void 0);
        tslib_1.__decorate([
            (0, decorators_1.SkipIf)(() => minMaxFields.length === 0, (0, graphql_1.Field)(() => MinMaxType, { nullable: true })),
            tslib_1.__metadata("design:type", Object)
        ], AggResponse.prototype, "min", void 0);
        tslib_1.__decorate([
            (0, decorators_1.SkipIf)(() => minMaxFields.length === 0, (0, graphql_1.Field)(() => MinMaxType, { nullable: true })),
            tslib_1.__metadata("design:type", Object)
        ], AggResponse.prototype, "max", void 0);
        AggResponse = tslib_1.__decorate([
            (0, graphql_1.ObjectType)(aggName)
        ], AggResponse);
        if (isShareable) {
            (0, decorators_1.setShareable)(AggResponse);
            (0, decorators_1.setShareable)(GroupType);
            (0, decorators_1.setShareable)(CountType);
            (0, decorators_1.setShareable)(FloatType);
            (0, decorators_1.setShareable)(MinMaxType);
        }
        return AggResponse;
    });
}
//# sourceMappingURL=aggregate-response.type.js.map