"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseRelativeDateFutureFormat = exports.parseRelativeDateFormat = exports.isIsoDate = exports.INVALID_RELATIVE_DATE_FORMAT_ERROR = exports.ACCEPTED_RELATIVE_DATE_FUTURE_VALUE = exports.ACCEPTED_RELATIVE_DATE_VALUE = exports.isRelativeDateFutureScalar = exports.isRelativeDateScalar = exports.RelativeDateFuturePrefix = exports.RelativeDatePrefix = exports.RelativeDateResolution = void 0;
var RelativeDateResolution;
(function (RelativeDateResolution) {
    RelativeDateResolution["minutes"] = "minutes";
    RelativeDateResolution["hours"] = "hours";
    RelativeDateResolution["days"] = "days";
    RelativeDateResolution["weeks"] = "weeks";
    RelativeDateResolution["months"] = "months";
    RelativeDateResolution["years"] = "years";
})(RelativeDateResolution || (exports.RelativeDateResolution = RelativeDateResolution = {}));
exports.RelativeDatePrefix = 'last';
exports.RelativeDateFuturePrefix = 'next';
const isRelativeDateScalar = (value) => value.startsWith(exports.RelativeDatePrefix);
exports.isRelativeDateScalar = isRelativeDateScalar;
const isRelativeDateFutureScalar = (value) => (0, exports.isRelativeDateScalar)(value) || value.startsWith(exports.RelativeDateFuturePrefix);
exports.isRelativeDateFutureScalar = isRelativeDateFutureScalar;
exports.ACCEPTED_RELATIVE_DATE_VALUE = 'an Epoch time (number), ISO Date string or a relative date value in the form of "last-[N]-[minutes | hours | days | weeks | months | years]"';
exports.ACCEPTED_RELATIVE_DATE_FUTURE_VALUE = 'an Epoch time (number), ISO Date string or a relative date value in the form of "[last | next]-[N]-[minutes | hours | days | weeks | months | years]"';
const INVALID_RELATIVE_DATE_FORMAT_ERROR = (future = false) => {
    throw new Error(`Invalid relative date value! Can only accept ${future ? exports.ACCEPTED_RELATIVE_DATE_FUTURE_VALUE : exports.ACCEPTED_RELATIVE_DATE_VALUE}`);
};
exports.INVALID_RELATIVE_DATE_FORMAT_ERROR = INVALID_RELATIVE_DATE_FORMAT_ERROR;
const validatePrefix = (val, isFuture) => {
    if (val === exports.RelativeDatePrefix)
        return;
    if (isFuture && exports.RelativeDateFuturePrefix)
        return;
    (0, exports.INVALID_RELATIVE_DATE_FORMAT_ERROR)(isFuture);
};
const parse = (value, isFuture) => {
    let span;
    const [prefix, spanStr, resolution] = value.split('-');
    validatePrefix(prefix, isFuture);
    try {
        span = parseInt(spanStr, 10);
    }
    catch (e) {
        (0, exports.INVALID_RELATIVE_DATE_FORMAT_ERROR)(isFuture);
    }
    if (!RelativeDateResolution[resolution])
        (0, exports.INVALID_RELATIVE_DATE_FORMAT_ERROR)(isFuture);
    return {
        resolution,
        span,
        isFuture: prefix === exports.RelativeDateFuturePrefix,
    };
};
const isIsoDate = (str) => {
    if (!/\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}.\d{3}Z/.test(str))
        return false;
    const d = new Date(str);
    return d instanceof Date && !Number.isNaN(d.getTime()) && d.toISOString() === str; // valid date
};
exports.isIsoDate = isIsoDate;
const parseRelativeDateFormat = (value) => parse(value, false);
exports.parseRelativeDateFormat = parseRelativeDateFormat;
const parseRelativeDateFutureFormat = (value) => parse(value, true);
exports.parseRelativeDateFutureFormat = parseRelativeDateFutureFormat;
//# sourceMappingURL=relative-date-scalar.helpers.js.map