"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RelativeDateScalar = void 0;
const tslib_1 = require("tslib");
const graphql_1 = require("@nestjs/graphql");
const graphql_2 = require("graphql");
const date_fns_1 = require("date-fns");
const relative_date_scalar_helpers_1 = require("./relative-date-scalar.helpers");
const getRelativeDate = (value) => {
    const { span, resolution } = (0, relative_date_scalar_helpers_1.parseRelativeDateFormat)(value);
    return (0, date_fns_1.sub)(new Date(), {
        [resolution]: span,
    });
};
let RelativeDateScalar = class RelativeDateScalar {
    constructor() {
        this.description = `Relative date, ${relative_date_scalar_helpers_1.ACCEPTED_RELATIVE_DATE_VALUE}`;
        this.name = 'RelativeDate';
    }
    parseValue(value) {
        if (typeof value === 'number')
            return new Date(value);
        if ((0, relative_date_scalar_helpers_1.isRelativeDateScalar)(value))
            return getRelativeDate(value);
        if ((0, relative_date_scalar_helpers_1.isIsoDate)(value))
            return new Date(value);
        return (0, relative_date_scalar_helpers_1.INVALID_RELATIVE_DATE_FORMAT_ERROR)();
    }
    serialize(value) {
        if (value instanceof Date)
            return value.toISOString();
        return new Date(value).toISOString();
    }
    parseLiteral(ast) {
        if (ast.kind === graphql_2.Kind.INT) {
            return new Date(ast.value);
        }
        if (ast.kind === graphql_2.Kind.STRING) {
            if ((0, relative_date_scalar_helpers_1.isRelativeDateScalar)(ast.value))
                return getRelativeDate(ast.value);
            if ((0, relative_date_scalar_helpers_1.isIsoDate)(ast.value))
                return new Date(ast.value);
        }
        return null;
    }
};
exports.RelativeDateScalar = RelativeDateScalar;
exports.RelativeDateScalar = RelativeDateScalar = tslib_1.__decorate([
    (0, graphql_1.Scalar)('RelativeDate', () => RelativeDateScalar)
], RelativeDateScalar);
//# sourceMappingURL=relative-date-scalar.type.js.map